package money.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;

public class BilancioFamiliare implements Bilancio
{
	private static final MovimentoComparator movimentoComparator = new MovimentoComparator();
	private HashMap<String, Conto> conti;
	private Esterno esterno;

	public BilancioFamiliare(Collection<Conto> conti, Collection<Movimento> movimenti)
	{
		this.conti = new HashMap<String, Conto>();
		for (Conto conto : conti)
		{
			this.conti.put(conto.getName(), conto);
		}
		this.esterno = new Esterno();
		this.conti.put(esterno.getName(), esterno);

		for (Movimento movimento : movimenti)
		{
			aggiungiMovimento(movimento);
		}
	}

	public Iterable<Conto> getConti()
	{
		return conti.values();
	}

	public Conto getConto(String nome)
	{
		return conti.get(nome);
	}

	public void aggiungiMovimento(Movimento m)
	{
		Conto sorgente = getConto(m.getSorgente());
		Conto destinazione = getConto(m.getDestinazione());
		if (sorgente == null || destinazione == null)
			throw new IllegalArgumentException("sorgente == null || destinazione == null");
		
		sorgente.aggiungiMovimento(m);
		destinazione.aggiungiMovimento(m);
	}

	@Override
	public double getTotaleEntrate(Date inizio, Date fine)
	// Fattibile in due modi: o sommando i movimenti da esterno di tutti i conti,
	// o pi semplicemente osservando (principio di pareggio del bilancio) che
	// il totale entrate della famiglia  uguale per definizione alle uscite
	// del conto "esterno"
	{
		double totale = 0;
		for (Movimento m : esterno.getMovimenti())
		{
			if (getConto(m.getSorgente()) == esterno && m.getData().getTime() >= inizio.getTime()
					&& m.getData().getTime() <= fine.getTime())
			{
				totale += m.getDenaroMosso();
			}
		}
		return totale;
	}
	// ALTERNATIVA PIU' RAPIDA E FURBA (Principio di Pareggio del Bilancio)
	// public double getTotaleEntrate(Date inizio, Date fine) {
	//	return conti.get("esterno").getTotaleUscite(inizio, fine);
	// }

	@Override
	public double getTotaleUscite(Date inizio, Date fine)
	// Fattibile in due modi: o sommando i movimenti verso esterno di tutti i conti,
	// o pi semplicemente osservando (principio di pareggio del bilancio) che
	// il totale uscite della famiglia  uguale per definizione alle entrate
	// del conto "esterno"
	{
		double totale = 0;
		for (Movimento m : esterno.getMovimenti(inizio, fine))
		{
			if (getConto(m.getDestinazione()) == esterno)
			{
				totale += m.getDenaroMosso();
			}
		}
		return totale;
	}
	// ALTERNATIVA PIU' RAPIDA E FURBA (Principio di Pareggio del Bilancio)
	// public double getTotaleUscite(Date inizio, Date fine) {
	//	return conti.get("esterno").getTotaleEntrate(inizio, fine);
	// }

	@Override
	public double getSaldo(Date fine)
	{
		return getTotaleEntrate(new Date(0), fine) - getTotaleUscite(new Date(0), fine);
	}

	public Collection<Movimento> getMovimenti(Date inizio, Date fine)
	{
		HashSet<Movimento> allMovimenti = new HashSet<Movimento>();
		for (Conto conto : getConti())
		{
			// add di un HashSet non consente duplicati: aggiunge solo gli elementi
			// non gi presenti			
			allMovimenti.addAll(conto.getMovimenti(inizio, fine));
		}
		ArrayList<Movimento> sortedMovimenti = new ArrayList<Movimento>(allMovimenti);
		Collections.sort(sortedMovimenti, movimentoComparator);
		return sortedMovimenti;
	}

	public Collection<Movimento> getMovimenti()
	{
		HashSet<Movimento> allMovimenti = new HashSet<Movimento>();
		for (Conto conto : getConti())
		{
			allMovimenti.addAll(conto.getMovimenti());
		}
		ArrayList<Movimento> sortedMovimenti = new ArrayList<Movimento>(allMovimenti);
		Collections.sort(sortedMovimenti, movimentoComparator);
		return sortedMovimenti;
	}

	public Collection<String> checkVincoli(Movimento m)
	{
		Conto sorgente = getConto(m.getSorgente());
		Conto destinazione = getConto(m.getDestinazione());
		if (sorgente == null || destinazione == null)
			throw new IllegalArgumentException("sorgente == null || destinazione == null");

		ArrayList<String> vincoliViolati = new ArrayList<String>();
		String error = sorgente.checkVincoli(m);
		if (error != null)
		{
			vincoliViolati.add(error);
		}
		error = destinazione.checkVincoli(m);
		if (error != null)
		{
			vincoliViolati.add(error);
		}
		return vincoliViolati;
	}

}
